/* tslint:disable:no-submodule-imports no-console */
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const colors = require("colors");
const fs = require("fs-extra");
const inquirer = require("inquirer");
const _ = require("lodash");
const pad = require("pad");
const path = require("path");
const rxjs_1 = require("rxjs");
exports.name = "init";
exports.alias = "";
exports.description = "create a new interaction.json";
const ALLOWED_ATTRIBUTES = [
    "platforms",
    "spreadsheets",
    "speechPath",
    "content",
    "contentPath",
    "viewsPath"
];
function action() {
    const interactionFile = path.join(process.cwd(), "interaction.json");
    let hasAnInteractionFile = false;
    try {
        // a path we KNOW is totally bogus and not a module
        hasAnInteractionFile = require(interactionFile);
    }
    catch (e) {
        if (e.code === "MODULE_NOT_FOUND") {
            hasAnInteractionFile = false;
        }
    }
    const observe = rxjs_1.Observable.create((obs) => {
        const override = (answers) => {
            return answers.interactionFile;
        };
        const onlyCommaAnswer = (input) => {
            return _.chain(input)
                .split(",")
                .map(_.trim)
                .compact()
                .value();
        };
        const noEmptyAnswer = (input) => !_.isEmpty(input);
        const mustBeSpreadsheetURLOrLength30 = (input) => {
            const result = _.chain(input)
                .split(",")
                .map(_.trim)
                .compact()
                .every((i) => {
                const matched = i.match(/docs\.google\.com\/spreadsheets\/d\/(.*)\/.*/);
                if (i.includes("docs.google.com/spreadsheets") && matched && _.isString(matched[1])) {
                    i = matched[1];
                }
                return i.length > 30;
            })
                .value();
            return noEmptyAnswer(input) && result
                ? true
                : "Insert a proper spreadsheet url or id eg. https://docs.google.com/spreadsheets/d/XXXXXXX/edit#gid=0";
        };
        const questions = [
            {
                type: "confirm",
                name: "interactionFile",
                message: `Do you want to override ${interactionFile}?`,
                when: () => hasAnInteractionFile,
                default: true
            },
            {
                type: "checkbox",
                name: "platforms",
                message: "Choose platforms",
                choices: ["alexa", "dialogflow"],
                when: override
            },
            {
                type: "input",
                name: "spreadsheets",
                message: "Specify all spreadsheets separated by comma (*)",
                validate: mustBeSpreadsheetURLOrLength30,
                filter: onlyCommaAnswer,
                when: override
            },
            {
                type: "input",
                name: "speechPath",
                message: "Specify folder path to save interaction model",
                default: "speech-assets",
                when: override
            },
            {
                type: "input",
                name: "contentPath",
                message: "Specify folder path to save all downloable content",
                when: override,
                default: "content"
            },
            {
                type: "input",
                name: "viewsPath",
                message: "Specify folder path to save views file",
                default: "/",
                when: override
            },
            {
                type: "confirm",
                name: "isCorrect",
                message: `is Correct?`,
                when: (answers) => {
                    if (override(answers)) {
                        printVariables(_.cloneDeep(answers));
                    }
                    return override(answers);
                },
                default: true
            }
        ];
        questions.map(q => obs.next(q));
        obs.complete();
    });
    function printVariables(answers) {
        return _.chain(answers)
            .toPairs()
            .filter(item => !!ALLOWED_ATTRIBUTES.find(i => i === item[0]))
            .map(item => {
            const key = item[0];
            const value = item[1];
            console.log(pad(colors.blue(key), 20), colors.grey(value));
            return item;
        })
            .compact()
            .fromPairs()
            .value();
    }
    function executePrompt() {
        return inquirer.prompt(observe).then((answers) => {
            if (answers.interactionFile && !answers.isCorrect) {
                return executePrompt();
            }
            if (hasAnInteractionFile && !answers.interactionFile) {
                answers = hasAnInteractionFile;
            }
            if (!answers.interactionFile && !answers.isCorrect) {
                printVariables(answers);
            }
            answers = _.pick(answers, ALLOWED_ATTRIBUTES);
            return fs.outputFileSync(path.join(process.cwd(), "interaction.json"), JSON.stringify(answers, null, 2), { flag: "w" });
        });
    }
    executePrompt();
}
exports.action = action;
//# sourceMappingURL=init.js.map